<?php

namespace GRIDBUILDERX;

// If this file is called directly, abort.
if (!defined('WPINC')) {
    die;
}


final class Builder
{

    # singleton
    private static $instance = null;

    public static function instance()
    {
        if (is_null(self::$instance)) {
            self::$instance = new self();
        }
        return self::$instance;
    }

    # prevent cloning
    private function __clone()
    {
        _doing_it_wrong(__FUNCTION__, esc_html__('Cloning not allowed', 'gbx'), '1.0.0');
    }

    # prevent unserializing
    public function __wakeup()
    {
        _doing_it_wrong(__FUNCTION__, esc_html__('Deserialization not alloed', 'gbx'), '1.0.0');
    }

    private function __construct()
    {
        $this->load_dependencies();

        add_action('plugins_loaded', [$this, 'awake_elementor_modules']);
        add_action('plugin_row_meta', [$this, 'gbx_plugin_row_meta'], 10, 2);
    }


    public function gbx_plugin_row_meta($links, $file)
    {
        if (GBX_PLUGIN_BASENAME === $file) {
            $row_meta = array(
                'docs'    => '<a href="' . esc_url('https://gridbuilderx.com/docs/') . '" aria-label="' . esc_attr__('View Grid Builder X Documentation', 'gbx') . '" target="_blank">' . esc_html__('Docs', 'gbx') . '</a>',
            );

            return array_merge($links, $row_meta);
        }

        return (array) $links;
    }


    // load the dependencies
    private function load_dependencies()
    {
        // load the query builder
        require_once GBX_PLUGIN_DIR . 'class/ajax-callbacks.php';
        require_once GBX_PLUGIN_DIR . 'utils/class-utils.php';
        require_once GBX_PLUGIN_DIR . 'utils/class-layout.php';
        require_once GBX_PLUGIN_DIR . 'utils/class-layout-builder.php';
    }

    public function awake_elementor_modules()
    {
        //load plugin text domain
        load_plugin_textdomain('gbx', false, GBX_PLUGIN_DIR . 'languages/');

        if (did_action('elementor/loaded')) {
            add_action('elementor/preview/enqueue_styles', function () {
                wp_enqueue_style('gbx-editor-only', GBX_PLUGIN_URL . 'assets/css/editor-only-style.css', GBX_VERSION, false);
            });

            //scripts for preview
            add_action('elementor/preview/enqueue_scripts', function () {
                wp_enqueue_script('gbx-editor-only', GBX_PLUGIN_URL . 'assets/js/editor-only-script.js', ['jquery'], GBX_VERSION, true);
            });
        }
        // search by title only filter
        add_filter('posts_where', function ($where, $wp_query) {
            global $wpdb;

            if ($search_term = $wp_query->get('gbx_search_prod_title')) {
                $search_term = sanitize_text_field($search_term);
                $where .= ' AND ' . $wpdb->posts . '.post_title LIKE \'%' . esc_sql($wpdb->esc_like($search_term)) . '%\'';
            }

            return $where;
        }, 10, 2);

        # check if elementor is active
        if (!did_action('elementor/loaded')) {
            # add admin notice
            add_action('admin_notices', function () {
?>
                <div class="notice notice-warning is-dismissible">
                    <p><?php esc_html_e('Grid Builder X requires Elementor to be installed and activated.', 'gbx'); ?></p>
                </div>
            <?php
            });
            return;
        }



        # check for required elementor version
        if (!version_compare(ELEMENTOR_VERSION, GBX_MINIMUM_ELEMENTOR_VERSION, '>=')) {

            # add admin notice
            add_action('admin_notices', function () {
            ?>
                <div class="notice notice-warning is-dismissible">
                    <p><?php esc_html_e('Grid Builder X requires Elementor version 2.0.0 or greater.', 'gbx'); ?></p>
                </div>
            <?php
            });
            return;
        }

        # check for required php version
        if (version_compare(PHP_VERSION, GBX_MINIMUM_PHP_VERSION, '<')) {

            # add admin notice
            add_action('admin_notices', function () {
            ?>
                <div class="notice notice-warning is-dismissible">
                    <p><?php esc_html_e('Grid Builder X requires PHP version 5.6 or greater.', 'gbx'); ?></p>
                </div>
<?php
            });
            return;
        }

        # add category
        add_action('elementor/elements/categories_registered', function ($elements_manager) {
            $elements_manager->add_category(
                'gbx_category',
                [
                    'title' => esc_html__('Grid Builder X', 'gbx'),
                    'icon' => 'fa fa-plug',
                ]
            );
        });


        add_action('elementor/widgets/register', [$this, 'init_elementor_widgets']);
        add_action('elementor/controls/register', [$this, 'init_elementor_controls']);
        require_once GBX_PLUGIN_DIR . 'utils/class-image-render.php';
    }

    /* 
    * init elementor widgets
    * @param $widgets_manager
    * @return void
    * @since 1.0.0
    */
    public function init_elementor_widgets($widgets_manager)
    {
        require_once GBX_PLUGIN_DIR . 'widgets/class-widget-grid-builder.php';
        $widgets_manager->register(new \GRIDBUILDERX\Widgets\Grid_Builder_X());

        require_once GBX_PLUGIN_DIR . 'widgets/class-widget-pagination.php';
        $widgets_manager->register(new \GRIDBUILDERX\Widgets\Grid_Builder_Pagination());

        require_once GBX_PLUGIN_DIR . 'widgets/class-widget-filtering.php';
        $widgets_manager->register(new \GRIDBUILDERX\Widgets\Grid_Builder_Filtering());
    }


    /*
    * init elementor controls
    * @param $control_manager
    * @return void
    * @since 1.0.0
    */

    public function init_elementor_controls($control_manager)
    {
        require_once GBX_PLUGIN_DIR . 'controls/class-control-dynamic-condition.php';
        $control_manager->register(new \GRIDBUILDERX\Controls\Dynamic_Field_Select_Control());

        require_once GBX_PLUGIN_DIR . 'controls/class-control-taxquery.php';
        $control_manager->register(new \GRIDBUILDERX\Controls\Args_Builder_Control());

        require_once GBX_PLUGIN_DIR . 'controls/class-control-search-posts.php';
        $control_manager->register(new \GRIDBUILDERX\Controls\Control_Search_Posts());

        require_once GBX_PLUGIN_DIR . 'controls/class-control-search-posts.php';
        $control_manager->register(new \GRIDBUILDERX\Controls\Control_Search_Posts());

        //taxonomy-filter-select
        require_once GBX_PLUGIN_DIR . 'controls/class-control-filter-taxonomy-searcher.php';
        $control_manager->register(new \GRIDBUILDERX\Controls\Filter_Taxonomy_Search_Control());
    }
}
